﻿using Infragistics.Win;
using Infragistics.Win.UltraWinToolbars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace UltraToolbars_Header_Button
{
    internal class RibbonCaptionAreaButtonCreationFilter : IUIElementCreationFilter
    {
        private Size buttonSize = new Size(30, 22);
        private string buttonContext = "A";

        void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
        {
            if (parent is RibbonCaptionAreaUIElement)
            {
                // See if the button element already exists in the collection.
                var myRibbonCaptionButtonUIElement = parent.GetDescendant(typeof(MyRibbonCaptionButtonUIElement), buttonContext) as MyRibbonCaptionButtonUIElement;
                if (null == myRibbonCaptionButtonUIElement)
                {
                    myRibbonCaptionButtonUIElement = new MyRibbonCaptionButtonUIElement(parent, buttonContext);
                    parent.ChildElements.Add(myRibbonCaptionButtonUIElement);
                }

                // Find the x coord of the left-most system button.
                int x = parent.Rect.Right;
                foreach (var ribbonCaptionButtonUIElement in parent.ChildElements.OfType<RibbonCaptionButtonUIElement>()) 
                {
                    x = Math.Min(x, ribbonCaptionButtonUIElement.Rect.X);
                }

                Rectangle parentRectInsideBorders = parent.RectInsideBorders;

                Rectangle buttonRect = new Rectangle(
                    x - buttonSize.Width,
                    parentRectInsideBorders.Y,
                    buttonSize.Width,
                    buttonSize.Height
                    );

                DrawUtility.AdjustVAlign(VAlign.Middle, ref buttonRect, parentRectInsideBorders);
                myRibbonCaptionButtonUIElement.Rect = buttonRect;
            }
        }

        bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
        {
            // Do nothing. 
            return false;
        }
    }   

    public class MyRibbonCaptionButtonUIElement : ButtonUIElementBase
    {
        #region Member Variables

        private ImageUIElement imageElement;
        private Color cachedImageForeColor = Color.Empty;
        private bool isActiveTool;

        private Image image;

        #endregion //Member Variables

        #region Constructor
        public MyRibbonCaptionButtonUIElement(UIElement parent, string context) : base(parent)
        {
            this.PrimaryContext = context;

            this.image = DummyDataCreator.GetTextBitmap(context.ToCharArray().First().ToString());
        }
        #endregion //Constructor

        #region Properties       

        #region ImageElement

        private ImageUIElement ImageElement
        {
            get
            {
                if (this.imageElement == null)
                    this.imageElement = new ImageUIElement(this, null);
                return this.imageElement;
            }
        }

        #endregion //ImageElement
       
        #endregion //Properties

        #region Base class overrides

        #region ButtonClick

        /// <summary>
        /// Invoked when the button has been clicked.
        /// </summary>
        /// <returns>True to prevent the base implementation from being invoked.</returns>
        protected override bool ButtonClick()
        {
            MessageBox.Show("My button was clicked.");
            return true;
        }
        #endregion //ButtonClick

        #region ButtonStyle

        /// <summary>
        /// Returns the style of the button
        /// </summary>
        public override UIElementButtonStyle ButtonStyle
        {
            get
            {
                return UIElementButtonStyle.Office2013Button;
            }
        }
        #endregion //ButtonStyle

        #region ClipSelf

        /// <summary>
        /// Overridden. Indicates if the elements rendering should be limited to its specified region.
        /// </summary>
        protected override bool ClipSelf
        {
            get
            {
                return true;
            }
        }

        #endregion //ClipSelf        

        
        protected override void DrawImage(ref UIElementDrawParams drawParams)
        {
            UltraToolbarsManager tbm = drawParams.ControlElement.UltraControl as UltraToolbarsManager;

            if (null != tbm)
            {
                Image img = drawParams.AppearanceData.GetImage(null);
                bool useDefaultImage = (img == null);
                
                if (null != img)
                {
                    using (ImageAttributes ia = new ImageAttributes())
                    {
                        if (useDefaultImage)
                        {
                            ColorMap[] maps = new ColorMap[1];
                            maps[0] = new ColorMap();
                            maps[0].OldColor = Color.Magenta;
                            maps[0].NewColor = drawParams.AppearanceData.ForeColor;
                            ia.SetRemapTable(maps);
                        }

                        Rectangle outerRect = this.RectInsideButtonBorders;
                        Size imgSize = img.Size;

                        Rectangle imageRect = new Rectangle(
                            outerRect.X + ((outerRect.Width - imgSize.Width) / 2),
                            outerRect.Y + ((outerRect.Height - imgSize.Height) / 2),
                            imgSize.Width,
                            imgSize.Height);

                        if ((this.ButtonState & UIElementButtonState.Inactive) == 0)
                            drawParams.DrawImage(img, imageRect, true, ia);
                        else
                        {
                            using (Image disabledImage = DrawUtility.CreateDisabledImage(img, Color.Transparent, ia))
                                drawParams.DrawImage(disabledImage, imageRect, true, ia);
                        }
                    }
                }
            }
        }

        #region InitAppearance

        /// <summary>
        /// Initializes the appearance structure for this element.
        /// </summary>
        /// <param name="appearanceData">The appearance structure to initialize.</param>
        /// <param name="requestedProps">The properties that are needed.</param>
        protected override void InitAppearance(ref AppearanceData appearanceData, ref AppearancePropFlags requestedProps)
        {
            bool drawToolAsActive = this.isActiveTool;
            UIElementButtonState buttonState = this.ButtonState;

            if (AppearancePropFlags.Image == (requestedProps & AppearancePropFlags.Image))
            {
                appearanceData.Image = this.image;
                requestedProps &= ~AppearancePropFlags.Image;
            }

            if (AppearancePropFlags.ImageHAlign == (requestedProps & AppearancePropFlags.ImageHAlign))
            {
                appearanceData.ImageHAlign = HAlign.Center;
                requestedProps &= ~AppearancePropFlags.ImageHAlign;
            }

            if (AppearancePropFlags.ImageVAlign == (requestedProps & AppearancePropFlags.ImageVAlign))
            {
                appearanceData.ImageVAlign = VAlign.Middle;
                requestedProps &= ~AppearancePropFlags.ImageVAlign;
            }

            if (AppearancePropFlags.ForeColor == (requestedProps & AppearancePropFlags.ForeColor))
            {
                appearanceData.ForeColor = Color.FromArgb(198, 198, 198);
                requestedProps &= ~AppearancePropFlags.ForeColor;
            }

            if (AppearancePropFlags.ThemedElementAlpha == (requestedProps & AppearancePropFlags.ThemedElementAlpha))
            {
                appearanceData.ThemedElementAlpha = Alpha.Transparent;
                requestedProps &= ~AppearancePropFlags.ThemedElementAlpha;
            }

            base.InitAppearance(ref appearanceData, ref requestedProps);           
        }

        #endregion //InitAppearance

        #region GetNonClientHitTestCode

        internal const int HTCLIENT = 1;

        /// <summary>
        /// Gets the WM_NCHITTEST code for special ui elements that simulate non-client areas, e.g. caption, resizing border etc. 
        /// </summary>
        /// <param name="point">The location, in client coordinates, where the hit test should occur.</param>
        /// <returns>The WM_NCHITTEST code which which indicates how the element should act at the specified location.</returns>
        [EditorBrowsable(EditorBrowsableState.Never)]
        public override int GetNonClientHitTestCode(Point point)
        {
            return HTCLIENT;
        }

        #endregion //GetNonClientHitTestCode	

        #region OnMouseEnter

        /// <summary>
        /// Overrides the OnMouseEnter to set the tool associated with this element as the active tool.
        /// </summary>
        protected override void OnMouseEnter()
        {
            this.isActiveTool = true;
            base.OnMouseEnter();
        }

        #endregion // OnMouseEnter

        #region OnMouseHover

        /// <summary>
        /// Overridden. Invoked when the mouse hovers over the button
        /// </summary>
        protected override void OnMouseHover()
        {
            // This would be the place to show a tooltip.
            base.OnMouseHover();
        }
        #endregion //OnMouseHover

        #region OnMouseLeave

        /// <summary>
        /// Overridden. Invoked when the mouse leaves the bounds of the tool
        /// </summary>
        protected override void OnMouseLeave()
        {
            // This would be the place to hide the tooltip.

            this.isActiveTool = false;
            base.OnMouseLeave();
        }
        #endregion //OnMouseLeave

        #region Region

        /// <summary>
		/// Overridden. Returns the region to which the button element should clip its rendering.
		/// </summary>
		public override Region Region
        {
            get
            {
                return DrawUtility.CreateBorderRegion(this.Rect, UIElementBorderStyle.None, Corners.All);
            }
        }
        #endregion //Region      

        #region WantsMouseHoverNotification

        /// <summary>
        /// Returns True if this element wants to be notified when the mouse hovers over it.
        /// </summary>
        protected override bool WantsMouseHoverNotification
        {
            get
            {
                return true;
            }
        }
        #endregion //WantsMouseHoverNotification

        #region IsButtonStyleMouseActive

        /// <summary>
        /// Overriden.
        /// </summary>
        protected override bool IsButtonStyleMouseActive
        {
            get
            {
                return true;
            }
        }

        #endregion //IsButtonStyleMouseActive

        #endregion // Base class overrides
    }


}
